@@ -4,7 +4,7 @@
 
 /* Copyright Philip Hazel 2021 */
 /* This file created: February 2021 */
-/* This file last modified: May 2022 */
+/* This file last modified: December 2023 */
 
 #include "pmw.h"
 
@@ -16,8 +16,8 @@ new operators into the check_ptr() function. */
 enum {
   dr_accleft, dr_add, dr_and,
   dr_barnumber, dr_bra,
-  dr_copy, dr_currentgray, dr_currentlinewidth, dr_currentpoint, dr_curveto,
-    dr_cvs,
+  dr_copy, dr_cos, dr_currentcolor, dr_currentdash, dr_currentgray,
+    dr_currentlinewidth, dr_currentpoint, dr_curveto, dr_cvs,
   dr_def, dr_div, dr_draw, dr_dup,
   dr_end, dr_eq, dr_exch, dr_exit,
   dr_false, dr_fill, dr_fillretain, dr_fontsize,
@@ -26,16 +26,16 @@ enum {
   dr_if, dr_ifelse,
   dr_jump,
   dr_ket,
-  dr_le, dr_leftbarx, dr_linebottom,
-  dr_linelength, dr_lineto, dr_linetop, dr_loop, dr_lt,
+  dr_le, dr_leftbarx, dr_linebottom, dr_linelength, dr_lineto, dr_linetop,
+    dr_loop, dr_lt,
   dr_magnification, dr_moveto, dr_mul,
   dr_ne, dr_neg, dr_not, dr_number,
   dr_or, dr_originx, dr_originy,
   dr_pagelength, dr_pagenumber, dr_pop, dr_pstack,
   dr_rcurveto, dr_repeat, dr_rlineto, dr_rmoveto, dr_roll,
-  dr_setgray, dr_setlinewidth, dr_show, dr_stavesize,
-  dr_stavespace, dr_stavestart, dr_stembottom, dr_stemtop,
-  dr_stringwidth, dr_stroke, dr_sub, dr_systemdepth,
+  dr_setcolor, dr_setdash, dr_setgray, dr_setlinewidth, dr_show, dr_sin,
+    dr_sqrt, dr_stavesize, dr_stavespace, dr_stavestart, dr_stembottom,
+    dr_stemtop, dr_stringwidth, dr_stroke, dr_sub, dr_systemdepth,
   dr_text, dr_topleft, dr_translate, dr_true,
   dr_varname, dr_varref,
   dr_xor
@@ -61,6 +61,9 @@ static uint32_t stack_rqd[] = {
   0u,                 /* barnumber */
   0u,                 /* bra */
   0x00000001u,        /* copy */
+  0x00000002u,        /* cos */
+  0u,                 /* currentcolor */
+  0u,                 /* currentdash */
   0u,                 /* currentgray */
   0u,                 /* currentlinewidth */
   0u,                 /* currentpoint */
@@ -118,9 +121,13 @@ static uint32_t stack_rqd[] = {
   0x00000022u,        /* rlineto */
   0x00000022u,        /* rmoveto */
   0x00000022u,        /* roll */
+  0x00000222u,        /* setcolor */
+  0x00000022u,        /* setdash */
   0x00000002u,        /* setgray */
   0x00000002u,        /* setlinewidth */
   0x00000003u,        /* show */
+  0x00000002u,        /* sin */
+  0x00000002u,        /* sqrt */
   0u,                 /* stavesize */
   0u,                 /* stavespace */
   0u,                 /* stavestart */
@@ -146,7 +153,8 @@ static uint32_t stack_rqd[] = {
 *************************************************/
 
 static BOOL currentpoint;
-static int gray;
+static int32_t colour[3];
+static int32_t dash[2];
 static int level;
 static int xp, yp, cp;
 static usint next_variable = 0;
@@ -178,7 +186,12 @@ static draw_op draw_operators[] = {
   { "and",          dr_and },
   { "barnumber",    dr_barnumber },
   { "copy",         dr_copy },
+  { "cos",          dr_cos },
+  { "currentcolor", dr_currentcolor },
+  { "currentcolour",dr_currentcolor },
+  { "currentdash",  dr_currentdash },
   { "currentgray",  dr_currentgray },
+  { "currentgrey",  dr_currentgray },
   { "currentlinewidth", dr_currentlinewidth },
   { "currentpoint", dr_currentpoint },
   { "curveto",      dr_curveto },
@@ -232,9 +245,15 @@ static draw_op draw_operators[] = {
   { "rlineto",      dr_rlineto },
   { "rmoveto",      dr_rmoveto },
   { "roll",         dr_roll },
+  { "setcolor",     dr_setcolor },
+  { "setcolour",    dr_setcolor },
+  { "setdash",      dr_setdash },
   { "setgray",      dr_setgray },
+  { "setgrey",      dr_setgray },
   { "setlinewidth", dr_setlinewidth },
   { "show",         dr_show },
+  { "sin",          dr_sin },
+  { "sqrt",         dr_sqrt },
   { "stavesize",    dr_stavesize },
   { "stavespace",   dr_stavespace },
   { "stavestart",   dr_stavestart },
@@ -287,7 +306,8 @@ for (int i = 0; i < out_drawstackptr; i++)
 
     if (d->size != 0) eprintf("/s%d", d->size + 1);
     if (d->rotate != 0) eprintf("/rot%s", sff(d->rotate));
-    if ((d->flags & text_boxed) != 0) eprintf("/box");
+    if ((d->flags & text_boxrounded) != 0) eprintf("/rbox");
+      else if ((d->flags & text_boxed) != 0) eprintf("/box");
     if ((d->flags & text_centre) != 0) eprintf("/c");
     if ((d->flags & text_endalign) != 0) eprintf("/e");
     if ((d->flags & text_ringed) != 0) eprintf("/ring");
@@ -409,6 +429,13 @@ while (read_c == '/')
       if (read_expect_integer(&rotate, TRUE, TRUE)) textptr->rotate = rotate;
       break;
       }
+    else if (Ustrncmp(main_readbuffer + read_i, "box", 3) == 0)
+      {
+      read_i += 3;
+      read_nextc();
+      textptr->flags |= text_boxed | text_boxrounded;
+      break;
+      }
     else if (Ustrncmp(main_readbuffer + read_i, "ing", 3) == 0)
       {
       read_i += 3;
@@ -733,7 +760,9 @@ if (last == NULL) out_overdraw = new; else
 
 new->next = NULL;
 new->texttype = FALSE;
-new->d.g.gray = gray;
+memcpy(new->d.g.colour, colour, 3 * sizeof(int32_t));
+new->d.g.dash[0] = dash[0];
+new->d.g.dash[1] = dash[1];
 new->d.g.linewidth = thickness;
 new->d.g.ystave = out_ystave;
 
@@ -791,6 +820,9 @@ while (pp != p && pp->d.val != dr_end)
     case dr_barnumber:
     case dr_bra:
     case dr_copy:
+    case dr_cos:
+    case dr_currentcolor:
+    case dr_currentdash:
     case dr_currentgray:
     case dr_currentlinewidth:
     case dr_currentpoint:
@@ -845,9 +877,13 @@ while (pp != p && pp->d.val != dr_end)
     case dr_rlineto:
     case dr_rmoveto:
     case dr_roll:
+    case dr_setcolor:
+    case dr_setdash:
     case dr_setgray:
     case dr_setlinewidth:
     case dr_show:
+    case dr_sin:
+    case dr_sqrt:
     case dr_stavesize:
     case dr_stavespace:
     case dr_stavestart:
@@ -1046,9 +1082,34 @@ while (p->d.val != dr_end)
       }
     break;
 
+    case dr_currentcolor:
+    draw_stack[out_drawstackptr].dtype = dd_number;
+    draw_stack[out_drawstackptr++].d.val = colour[0];
+    draw_stack[out_drawstackptr].dtype = dd_number;
+    draw_stack[out_drawstackptr++].d.val = colour[1];
+    draw_stack[out_drawstackptr].dtype = dd_number;
+    draw_stack[out_drawstackptr++].d.val = colour[2];
+    break;
+
+    case dr_currentdash:
+    draw_stack[out_drawstackptr].dtype = dd_number;
+    draw_stack[out_drawstackptr++].d.val = dash[0];
+    draw_stack[out_drawstackptr].dtype = dd_number;
+    draw_stack[out_drawstackptr++].d.val = dash[1];
+    break;
+
+    /* The formula for converting a non-grey colour to grey is taken from the
+    NTSC video standard that is used to convert colour television to black and
+    white, as described in the PostScript documentation. The formula is:
+    gray = 0.3*red + 0.59*green + 0.11*blue. */
+
     case dr_currentgray:
     draw_stack[out_drawstackptr].dtype = dd_number;
-    draw_stack[out_drawstackptr++].d.val = gray;
+    if (colour[0] == colour[1] && colour[1] == colour[2])
+      draw_stack[out_drawstackptr++].d.val = colour[0];
+    else
+      draw_stack[out_drawstackptr++].d.val =
+        (30 * colour[0] + 59 * colour[1] + 11 * colour[2])/100;
     break;
 
     case dr_currentlinewidth:
@@ -1105,7 +1166,8 @@ while (p->d.val != dr_end)
     break;
 
     case dr_div:
-    if (draw_stack[out_drawstackptr-1].d.val == 0) draw_error(ERR155, "", t);
+    if (draw_stack[out_drawstackptr-1].d.val == 0)
+      draw_error(ERR155, "division by zero", t);
     draw_stack[out_drawstackptr-2].d.val =
       mac_muldiv(draw_stack[out_drawstackptr-2].d.val, 1000,
         draw_stack[out_drawstackptr-1].d.val);
@@ -1427,9 +1489,23 @@ while (p->d.val != dr_end)
       }
     break;
 
+    case dr_setcolor:
+    colour[2] = draw_stack[--out_drawstackptr].d.val;
+    colour[1] = draw_stack[--out_drawstackptr].d.val;
+    colour[0] = draw_stack[--out_drawstackptr].d.val;
+    ps_setcolour(colour);
+    break;
+
+    case dr_setdash:
+    dash[1] = draw_stack[--out_drawstackptr].d.val;
+    dash[0] = draw_stack[--out_drawstackptr].d.val;
+    ps_setdash(dash[0], dash[1]);
+    break;
+
     case dr_setgray:
-    gray = draw_stack[--out_drawstackptr].d.val;
-    ps_setgray(gray);
+    colour[0] = draw_stack[--out_drawstackptr].d.val;
+    colour[1] = colour[2] = colour[0];
+    ps_setcolour(colour);
     break;
 
     case dr_setlinewidth:
@@ -1437,6 +1513,25 @@ while (p->d.val != dr_end)
       out_stavemagn, 1000);
     break;
 
+    case dr_sin:
+    draw_stack[out_drawstackptr-1].d.val =
+      (int)(sin((double)(draw_stack[out_drawstackptr-1].d.val)*3.14159/180000.0)
+        * 1000.0);
+    break;
+
+    case dr_cos:
+    draw_stack[out_drawstackptr-1].d.val =
+      (int)(cos((double)(draw_stack[out_drawstackptr-1].d.val)*3.14159/180000.0)
+        * 1000.0);
+    break;
+
+    case dr_sqrt:
+    if (draw_stack[out_drawstackptr-1].d.val < 0)
+      draw_error(ERR155, "negative argument for square root", t);
+    else draw_stack[out_drawstackptr-1].d.val =
+      (int)(sqrt((double)(draw_stack[out_drawstackptr-1].d.val)/1000.0)*1000.0);
+    break;
+
     case dr_stringwidth:
     case dr_show:
       {
@@ -1467,7 +1562,6 @@ while (p->d.val != dr_end)
         int32_t xx = x[xp-1] + d->xdelta;
         int32_t yy = y[yp-1] + d->ydelta;
         uint32_t flags = d->flags;
-        uint32_t boxring = flags & (text_boxed | text_ringed);
 
         if (!currentpoint) draw_error(ERR153, "show", t);
 
@@ -1502,7 +1596,8 @@ while (p->d.val != dr_end)
           new->next = NULL;
           new->texttype = TRUE;
           new->d.t.text = d->text;
-          new->d.t.boxring = boxring;
+          new->d.t.flags = flags;
+          memcpy(new->d.t.colour, colour, 3 * sizeof(int32_t));
           new->d.t.xx = xx;
           new->d.t.yy = out_ystave - yy;
           new->d.t.fdata = *fdata;
@@ -1513,7 +1608,7 @@ while (p->d.val != dr_end)
             }
           }
 
-        else out_string(d->text, fdata, xx, out_ystave - yy, boxring);
+        else out_string(d->text, fdata, xx, out_ystave - yy, flags);
         }
       }
     break;
@@ -1623,16 +1718,23 @@ out_dodraw(tree_node *t, drawitem *args, BOOL overflag)
 int32_t x[100];
 int32_t y[100];
 int c[100];
+int32_t save_colour[3];
 
 if (args != NULL)
   for (int i = 1; i <= args[0].d.val; i++)
     draw_stack[out_drawstackptr++] = args[i];
 
-xp = yp = cp = level = gray = 0;
+xp = yp = cp = level = colour[0] = colour[1] = colour[2] =
+  dash[0] = dash[1] = 0;
 currentpoint = FALSE;
+ps_getcolour(save_colour);
 ps_setgray(0);
+ps_setdash(0,0);
+ps_setcapandjoin(0);
+draw_thickness = 500;
 (void)sub_draw(t, NULL, x, y, c, overflag);
-ps_setgray(0);
+ps_setcolour(save_colour);
+ps_setcapandjoin(0);
 }
 
 
