@@ -2,17 +2,17 @@
 *              Main header for PMW               *
 *************************************************/
 
-/* Copyright Philip Hazel 2022 */
+/* Copyright Philip Hazel 2023 */
 
 /* PMW rewrite project started: December 2020 */
 /* This file created: December 2020 */
-/* This file last modified: July 2022 */
+/* This file last modified: December 2023 */
 
 /* This file is included by all the other sources except rdargs.c. */
 
-#define PMW_VERSION "5.20"
-#define PMW_DATE    "28-June-2022"
-#define COPYRIGHT   "Copyright (c) Philip Hazel 2022"
+#define PMW_VERSION "5.22"
+#define PMW_DATE    "16-October-2023"
+#define COPYRIGHT   "Copyright (c) Philip Hazel 2023"
 
 /* Standard C headers */
 
@@ -83,6 +83,7 @@
 #define PAGE_LEFTBARSPACE           8000  /* Space at left of lefthand bar */
 #define READ_FOOTINGSIZE            8000  /* Default size ) for headings */
 #define READ_PAGEHEADINGSIZE       10000  /* Default size ) there is a list */
+#define STEMCENTADJUST              2545  /* Adjustment for central stems */
 #define STEMSTACKSIZE                 50  /* Size of pending stem stack */
 #define STRETCHTHRESHDEN               2  /* Currently fixed stretching */
 #define STRETCHTHRESHNUM               1  /*   threshold of 1/2 */
@@ -370,7 +371,7 @@ enum error_number {
   ERR140,ERR141,ERR142,ERR143,ERR144,ERR145,ERR146,ERR147,ERR148,ERR149,
   ERR150,ERR151,ERR152,ERR153,ERR154,ERR155,ERR156,ERR157,ERR158,ERR159,
   ERR160,ERR161,ERR162,ERR163,ERR164,ERR165,ERR166,ERR167,ERR168,ERR169,
-  ERR170,ERR171,ERR172,ERR173,ERR174,ERR175,ERR176
+  ERR170,ERR171,ERR172,ERR173,ERR174,ERR175,ERR176,ERR177,ERR178
 };
 
 /* Types of input file */
@@ -422,15 +423,16 @@ enum { lv_barcount, lv_repeatcount, lv_repeatptr, lv_newpage };
 #define text_baralign   0x00000008u  /* Align text at starting barline */
 #define text_barcentre  0x00000010u  /* Centre the text in the bar */
 #define text_boxed      0x00000020u  /* Enclose in a box */
-#define text_centre     0x00000040u  /* Centred  */
-#define text_endalign   0x00000080u  /* End aligned */
-#define text_fb         0x00000100u  /* Figured bass */
-#define text_followon   0x00000200u  /* Follow-on to previous */
-#define text_middle     0x00000400u  /* Put halfway betwen staves */
-#define text_rehearse   0x00000800u  /* Rehearsal mark */
-#define text_ringed     0x00001000u  /* Enclose in a ring */
-#define text_timealign  0x00002000u  /* Align with time signature */
-#define text_ul         0x00004000u  /* Underlay, but with text_above = overlay */
+#define text_boxrounded 0x00000040u  /* ... with rounded corners */
+#define text_centre     0x00000080u  /* Centred  */
+#define text_endalign   0x00000100u  /* End aligned */
+#define text_fb         0x00000200u  /* Figured bass */
+#define text_followon   0x00000400u  /* Follow-on to previous */
+#define text_middle     0x00000800u  /* Put halfway betwen staves */
+#define text_rehearse   0x00001000u  /* Rehearsal mark */
+#define text_ringed     0x00002000u  /* Enclose in a ring */
+#define text_timealign  0x00004000u  /* Align with time signature */
+#define text_ul         0x00008000u  /* Underlay; with text_above = overlay */
 
 /* Flags for stave name texts */
 
@@ -627,21 +629,18 @@ space in 32-bits. */
 #define nf_highdot     0x00020000u  /* Move space dot up to next space */
 #define nf_invert      0x00040000u  /* Print this note on other side of stem */
 #define nf_lowdot      0x00080000u  /* Print dots/plus below line */
-#define nf_nhcross     0x00100000u  /* Cross notehead */
-#define nf_nhharmonic  0x00200000u  /* Harmonic (diamond) notehead */
-#define nf_nopack      0x00400000u  /* Do not pack this rest bar */
-#define nf_noplay      0x00800000u  /* Do not play - note tied */
-#define nf_plus        0x01000000u  /* Augmentation plus */
-#define nf_restrep     0x02000000u  /* Rest displayed as repetition sign */
-#define nf_shortened   0x04000000u  /* This note's stem has been automatically shortened */
-#define nf_smallhead   0x08000000u  /* Small notehead */
-#define nf_stem        0x10000000u  /* This note has a stem */
-#define nf_stemup      0x20000000u  /* Stem direction */
-#define nf_tripletize  0x40000000u  /* Check note for tripletizing */
+#define nf_nopack      0x00100000u  /* Do not pack this rest bar */
+#define nf_noplay      0x00200000u  /* Do not play - note tied */
+#define nf_plus        0x00400000u  /* Augmentation plus */
+#define nf_restrep     0x00800000u  /* Rest displayed as repetition sign */
+#define nf_shortened   0x01000000u  /* This note's stem has been automatically shortened */
+#define nf_stem        0x02000000u  /* This note has a stem */
+#define nf_stemup      0x04000000u  /* Stem direction */
+#define nf_tripletize  0x08000000u  /* Check note for tripletizing */
+#define nf_stemcent    0x10000000u  /* Place stem central to note */
 
 #define nf_couple      (nf_coupleU+nf_coupleD)
 #define nf_dotted      (nf_dot+nf_dot2+nf_plus)
-#define nf_notrests    (nf_stemup|nf_smallhead|nf_headbra)
 
 /* Flags for the tie item (currently 8-bits) */
 
@@ -654,18 +653,23 @@ space in 32-bits. */
 #define tief_savedash   0x40u
 
 /* Notehead type options. If these are changed, the "headchars" table in
-setnote.c must also be changed. No stems are printed for values greater than or
-equal to nh_only. */
+setnote.c must also be changed. */
 
-enum { nh_normal,           /* conventional noteheads, with stems and beams */
-       nh_cross,            /* X noteheads, with stems and beams */
-       nh_harmonic,         /* diamond-shaped, with stems and beams */
-       nh_none,             /* no noteheads, just stems and beams */
-       nh_only,             /* conventional noteheads, no stems or beams */
-       nh_direct,           /* fancy 'w', no stems or beams */
+enum { nh_normal,           /* conventional noteheads */
+       nh_cross,            /* X noteheads */
+       nh_harmonic,         /* diamond-shaped */
+       nh_none,             /* no noteheads */
+       nh_direct,           /* fancy 'w' */
+       nh_circular,         /* special circular shape */
        nh_number            /* number of special noteheads values */
 };
 
+/* The mask is for the bottom bits of a note's noteheadstyle field, which
+contain one of the values above, leaving the rest for flags. */
+
+#define nh_mask 0x07u
+#define nhf_smallhead   0x80u         /* Small notehead */
+
 /* Flags for accents on a note. Ornaments are held separately, as individual
 numbers less than 256, but both accent and ornament encodings are listed in a
 single table. This table relies on accent bit values being greater than 255; in
