@@ -4,7 +4,7 @@
 
 /* Copyright Philip Hazel 2022 */
 /* This file created: March 2021 */
-/* This file last modified: May 2022 */
+/* This file last modified: October 2023 */
 
 /* This file contains the code for reading one note in PMW notation. */
 
@@ -187,6 +187,10 @@ do_undoverlay(b_textstr **pp)
 {
 b_textstr *p = *pp;
 
+/* If there has been a [reset] and we haven't yet caught up, do nothing. */
+
+if (brs.barlength <= brs.maxbarlength) return;
+
 /* Loop for each "verse" */
 
 while (p != NULL)
@@ -1415,6 +1419,7 @@ for (;;)
   uint8_t acc;
   uint8_t acc_orig, char_orig;
   uint8_t transposedacc = ac_no;
+  uint8_t noteheadstyle = srs.noteheadstyle;
   BOOL    transposedaccforce = active_transposedaccforce;
   BOOL    note_set_taf = FALSE;
 
@@ -1851,6 +1856,14 @@ for (;;)
   if (srs.matchnum > 0 && (flags & nf_centre) == 0)
     pn_notelength = mac_muldiv(pn_notelength, srs.matchnum, srs.matchden);
 
+  /* If a whole bar rest, flag it for centring unless we are in an unchecked
+  bar. (If the rest was specified using the ! notation it will already be
+  flagged for centring. This code covers other cases.) We must do this before
+  checking the options so that \C\ can be used to turn off centring. */
+
+  if (pitch == 0 && pn_notelength == srs.required_barlength && brs.checklength)
+    flags |= nf_centre;
+
   /* Handle options enclosed in \...\ that follow a note. These include accents
   and ornaments, stem direction control, local level for rests, etc. */
 
@@ -1977,9 +1990,44 @@ for (;;)
 
         case 'n':
         read_nextc();
-        if (read_c == 'h') { flags |= nf_nhharmonic; read_nextc(); }
-          else if (read_c == 'x') { flags |= nf_nhcross; read_nextc(); }
-            else error_skip(ERR114, '\\');
+        if (read_c == 'o') flags &= ~nf_stem; else
+          {
+          noteheadstyle &= ~nh_mask;
+          flags |= nf_stem;
+          switch(read_c)
+            {
+            case 'c':
+            noteheadstyle |= nh_circular;
+            flags &= ~nf_stem;
+            break;
+
+            case 'd':
+            noteheadstyle |= nh_direct;
+            flags &= ~nf_stem;
+            break;
+
+            case 'h':
+            noteheadstyle |= nh_harmonic;
+            break;
+
+            case 'n':
+            noteheadstyle |= nh_normal;
+            break;
+
+            case 'x':
+            noteheadstyle |= nh_cross;
+            break;
+
+            case 'z':
+            noteheadstyle |= nh_none;
+            break;
+
+            default:
+            error(ERR8, "nc, nd, nh, nn, no, nx, or nz");
+            break;
+            }
+          }
+        read_nextc();
         break;
 
         case 's':
@@ -2019,7 +2067,7 @@ for (;;)
         else if (read_c == 'm')
           {
           read_nextc();
-          flags |= nf_smallhead;
+          noteheadstyle |= nhf_smallhead;
           }
         else if (read_c == 'p')   /* sp is an ornament */
           {
@@ -2140,12 +2188,13 @@ for (;;)
     if (read_c == '\\') read_nextc();
     }                                  /* end options handling */
 
-  /* Rests should not have ornaments except fermata or accidentals or certain
+  /* Rests should not have ornaments (except fermata) or accidentals or certain
   options. */
 
   if (pitch == 0 && (
       ((ornset & ~(1 << or_ferm)) != 0 && (flags & nf_hidden) == 0) ||
-      (flags & nf_notrests) != 0 ||
+      (flags & (nf_stemup|nf_headbra)) != 0 ||
+      (noteheadstyle & nhf_smallhead) != 0 ||
       acc != ac_no ||
       pn_stemforce != 0 ||
       (acflags & af_accents) != 0))
@@ -2153,7 +2202,7 @@ for (;;)
 
   /* A small notehead may not be specified for a grace note. */
 
-  if (pn_notelength == 0 && (flags & nf_smallhead) != 0) error(ERR123);
+  if (pn_notelength == 0 && (noteheadstyle & nhf_smallhead) != 0) error(ERR123);
 
   /* Accumulate bar length and check that chord notes are all same length. If
   not, and the current one is a crotchet (i.e. just a lower case letter) force
@@ -2228,13 +2277,6 @@ for (;;)
     ornament = or_unset;
     }
 
-  /* If a whole bar rest, flag it for centring unless we are in an unchecked
-  bar. (If the rest was specified as R! then it has already been marked as a
-  whole bar rest.) */
-
-  if (pitch == 0 && pn_notelength == srs.required_barlength && brs.checklength)
-    flags |= nf_centre;
-
   /* Now create a data block for the note or rest, saving the address of the
   first one in a chord for exact duplications ('x') and the first one in a
   non-p-duplicated chord for subsequent 'p' duplications, except in the case of
@@ -2263,6 +2305,7 @@ for (;;)
   noteptr->yextra = yextra;
   noteptr->accleft = accleft;
   noteptr->abspitch = abspitch;
+  noteptr->noteheadstyle = noteheadstyle;
   pn_chordcount++;
 
   /* If we are duplicating as a result of 'p', pn_inchord will be 0 because
