@@ -2,9 +2,9 @@
 *  PMW native stave directive reading functions  *
 *************************************************/
 
-/* Copyright Philip Hazel 2021 */
+/* Copyright Philip Hazel 2023 */
 /* This file created: February 2021 */
-/* This file last modified: May 2022 */
+/* This file last modified: December 2023 */
 
 #include "pmw.h"
 
@@ -165,6 +165,35 @@ if (size == 0 || --size >= UserFontSizes)
 
 
 
+/*************************************************
+*       Common function for reset and backup     *
+*************************************************/
+
+static void
+resetorbackup(const char *name, uint32_t whereto, BOOL reset)
+{
+b_resetstr *r = mem_get_item(sizeof(b_resetstr), b_reset);
+
+/* Do things that are otherwise done at bar end */
+
+if (srs.beaming) read_setbeamstems();
+if (brs.barlength > brs.maxbarlength) brs.maxbarlength = brs.barlength;
+
+/* Test for a valid reset/backup */
+
+if (!brs.resetOK)
+  error((brs.barlength == 0)? ERR105 : ERR106, name);
+    else if (brs.pletlen != 0 && reset) error(ERR107, name);
+
+/* We do the action anyway, to prevent spurious over-long bar errors. */
+
+if (reset) read_init_baraccs(read_baraccs, srs.key);
+brs.barlength = r->moff = whereto;
+brs.resetOK = FALSE;
+}
+
+
+
 /*************************************************
 *           Clef and octave setting              *
 *************************************************/
@@ -313,6 +342,20 @@ error_skip(ERR8, ']', "clef, key, or time setting");
 
 
 
+
+/*************************************************
+*                  Backup                         *
+*************************************************/
+
+static void
+p_backup(void)
+{
+if (srs.lastnoteptr == NULL) error(ERR177);
+  else resetorbackup("backup", brs.barlength - srs.lastnoteptr->length, FALSE);
+}
+
+
+
 /************************************************
 *           Barlinestyle                        *
 ************************************************/
@@ -980,26 +1023,40 @@ brs.nocount = TRUE;
 static void
 p_noteheads(void)
 {
-b_noteheadsstr *p = mem_get_item(sizeof(b_noteheadsstr), b_noteheads);
+usint value;
 
-if (dir->arg1 < nh_number) p->value = dir->arg1; else
+srs.noteflags |= nf_stem;   /* All except "only" and "direct" set a stem */
+
+if (dir->arg1 < nh_number)
+  {
+  value = dir->arg1;
+  }
+else
   {
   read_nextword();
-  if (Ustrcmp(read_wordbuffer, "only") == 0) p->value = nh_only;
-  else if (Ustrcmp(read_wordbuffer, "direct") == 0) p->value = nh_direct;
-  else if (Ustrcmp(read_wordbuffer, "normal") == 0) p->value = nh_normal;
-  else if (Ustrcmp(read_wordbuffer, "harmonic") == 0) p->value = nh_harmonic;
-  else if (Ustrcmp(read_wordbuffer, "cross") == 0) p->value = nh_cross;
-  else if (Ustrcmp(read_wordbuffer, "none") == 0) p->value = nh_none;
+  if (Ustrcmp(read_wordbuffer, "only") == 0)
+    {
+    value = srs.noteheadstyle & nh_mask;  /* Existing value */
+    srs.noteflags &= ~nf_stem;
+    }
+  else if (Ustrcmp(read_wordbuffer, "direct") == 0)
+    {
+    value = nh_direct;
+    srs.noteflags &= ~nf_stem;
+    }
+  else if (Ustrcmp(read_wordbuffer, "normal") == 0) value = nh_normal;
+  else if (Ustrcmp(read_wordbuffer, "harmonic") == 0) value = nh_harmonic;
+  else if (Ustrcmp(read_wordbuffer, "cross") == 0) value = nh_cross;
+  else if (Ustrcmp(read_wordbuffer, "none") == 0) value = nh_none;
+  else if (Ustrcmp(read_wordbuffer, "circular") == 0) value = nh_circular;
   else
     {
-    error(ERR8, "\"normal\", \"harmonic\", \"cross\", \"none\", \"only\", or \"direct\"");
-    p->value = nh_normal;
+    error(ERR8, "\"normal\", \"harmonic\", \"cross\", \"circular\", \"none\", \"only\", or \"direct\"");
+    value = nh_normal;
     }
   }
 
-if (p->value < nh_only) srs.noteflags |= nf_stem;
-  else srs.noteflags &= ~nf_stem;
+srs.noteheadstyle = (srs.noteheadstyle & ~nh_mask) | value;
 }
 
 
@@ -1157,23 +1214,7 @@ else srs.printpitch = read_stavepitch();
 static void
 p_reset(void)
 {
-(void)mem_get_item(sizeof(bstr), b_reset);
-
-/* Do things that are otherwise done at bar end */
-
-if (srs.beaming) read_setbeamstems();
-if (brs.barlength > brs.maxbarlength) brs.maxbarlength = brs.barlength;
-
-/* Test for a valid reset */
-
-if (!brs.resetOK)
-  error((brs.barlength == 0)? ERR105 : ERR106);
-    else if (brs.pletlen != 0) error(ERR107);
-
-/* We do the action anyway, to prevent spurious over-long bar errors. */
-
-read_init_baraccs(read_baraccs, srs.key);
-brs.barlength = 0;
+resetorbackup("reset", 0, TRUE);
 brs.resetOK = FALSE;
 }
 
@@ -1657,7 +1698,7 @@ while (read_c == '/')
           read_nextc();
           p->textflags |= text_boxed;
           }
-        else error(ERR8, "/box, /ring, or /s");
+        else error(ERR8, "/box");
         break;
 
         case 'd':
@@ -1691,7 +1732,13 @@ while (read_c == '/')
         break;
 
         case 'r':
-        if (Ustrncmp(main_readbuffer + read_i, "ing", 3) == 0)
+        if (Ustrncmp(main_readbuffer + read_i, "box", 3) == 0)
+          {
+          read_i += 3;
+          read_nextc();
+          p->textflags |= text_boxed | text_boxrounded;
+          }
+        else if (Ustrncmp(main_readbuffer + read_i, "ing", 3) == 0)
           {
           read_i += 3;
           read_nextc();
@@ -1705,7 +1752,7 @@ while (read_c == '/')
         break;
 
         default:
-        error(ERR8, "/box, /ring, or /s");
+        error(ERR8, "/box, /d, /l, /r, /rbox, /ring, /s, or /u");
         break;
         }
       }
@@ -1821,8 +1868,30 @@ p_stemlength(void)
 static void
 p_stems(void)
 {
-int8_t *p = (dir->arg1)? &srs.stemsdirection : &srs.tiesplacement;
+int8_t *p;
+
 read_nextword();
+
+if (dir->arg1)    /* "Stems" directive has additional settings */
+  {
+  if (Ustrcmp(read_wordbuffer, "central") == 0)
+    {
+    srs.noteflags |= nf_stemcent;
+    return;
+    }
+  if (Ustrcmp(read_wordbuffer, "beside") == 0)
+    {
+    srs.noteflags &= ~nf_stemcent;
+    return;
+    }
+
+  p = &srs.stemsdirection;
+  }
+
+else p = &srs.tiesplacement;
+
+/* Common to stems and ties */
+
 if (Ustrcmp(read_wordbuffer, "auto") == 0) *p = 0;
 else if (Ustrcmp(read_wordbuffer, "up") == 0 || Ustrcmp(read_wordbuffer, "above") == 0) *p = +1;
 else if (Ustrcmp(read_wordbuffer, "down") == 0 || Ustrcmp(read_wordbuffer, "below") == 0) *p = -1;
@@ -2111,6 +2180,7 @@ static dirstr read_stavedirlist[] = {
   { "all",            p_common,        b_all, TRUE },
   { "alto",           p_clef,          clef_alto, FALSE },
   { "assume",         p_assume,        0, TRUE },
+  { "backup",         p_backup,        0, TRUE },
   { "baritone",       p_clef,          clef_baritone, FALSE },
   { "barlinestyle",   p_barlinestyle,  0, TRUE },
   { "barnumber",      p_barnum,        0, TRUE },
@@ -2122,6 +2192,7 @@ static dirstr read_stavedirlist[] = {
   { "bottommargin",   p_pvalue,        b_pagebotmargin, TRUE },
   { "bowing",         p_bowing,        0, TRUE },
   { "breakbarline",   p_common,        b_breakbarline, TRUE },
+  { "c",              p_noteheads,     nh_circular, TRUE },
   { "cbaritone",      p_clef,          clef_cbaritone, FALSE },
   { "comma",          p_common,        b_comma, FALSE },
   { "contrabass",     p_clef,          clef_contrabass, FALSE },
