@@ -4,7 +4,7 @@
 
 /* Copyright Philip Hazel 2022 */
 /* This file created: May 2021 */
-/* This file last modified: July 2022 */
+/* This file last modified: December 2023 */
 
 #include "pmw.h"
 
@@ -17,6 +17,7 @@
 
 static BOOL ps_EPS = FALSE;
 static BOOL ps_slurA = FALSE;
+static BOOL ps_changecolour= FALSE;
 
 static uint32_t ps_caj;
 static int  ps_chcount;
@@ -30,7 +31,8 @@ static int32_t ps_fmatrix[6];
 static stavelist *ps_curlist;
 static uint32_t ps_curnumber;
 
-static int32_t  ps_gray;
+static int32_t  ps_wantcolour[3] = {0, 0, 0};
+static int32_t  ps_curcolour[3] = {0, 0, 0};
 static int32_t  ps_ymax;
 
 static uschar  *ps_IdStrings[font_tablen+1];
@@ -315,6 +317,24 @@ ps_chcount = (p[len-1] == '\n')? 0 : ps_chcount + len;
 
 
 
+/*************************************************
+*            Manage colour setting               *
+*************************************************/
+
+static void
+setcolour(void)
+{
+if (ps_wantcolour[0] == ps_wantcolour[1] && ps_wantcolour[1] == ps_wantcolour[2])
+  ps_printf(" %s Sg", sff(ps_wantcolour[0]));
+else
+  ps_printf(" %s %s %s Sc", sff(ps_wantcolour[0]), sff(ps_wantcolour[1]),
+    sff(ps_wantcolour[2]));
+memcpy(ps_curcolour, ps_wantcolour, 3 * sizeof(int32_t));
+ps_changecolour = FALSE;
+}
+
+
+
 /*************************************************
 *         Check whether font needs changing      *
 *************************************************/
@@ -605,11 +625,8 @@ if (main_righttoleft || ps_EPS)
     }
   }
 
-/* Generate the output. For a font that is neither standardly encoded, nor has
-a custom encoding, the values are less than 256. For a standardly encoded font
-they are less than LOWCHARLIMIT plus some extras. For a font with a custom
-encoding they are less than 512. Values above 255 use the second encoding of
-such fonts. */
+/* Generate the output. Values are always less than FONTWIDTHS_SIZE (512);
+those above 255 use the second font encoding. */
 
 for (p = s; *p != 0; p++)
   {
@@ -617,13 +634,10 @@ for (p = s; *p != 0; p++)
   uint32_t pc = c;           /* pc is the code value to print */
   BOOL extended = FALSE;
 
-  if ((fs->flags & ff_stdencoding) != 0 || fs->encoding != NULL)
+  if (c >= 256)
     {
-    if (c >= 256)
-      {
-      pc -= 256;
-      extended = TRUE;
-      }
+    pc -= 256;
+    extended = TRUE;
     }
 
   /* Change between base and extended font if necessary */
@@ -725,10 +739,10 @@ Special action is needed for the music font. The PostScript Type 1 font
 contains a few characters that output nothing but whose horizontal spacial
 increment is negative and also some characters (both blank and non-blank) with
 vertical movement - features which are useful for building composite things
-from strings in this font. Annoyingly, OpenType fonts do not permit negative
-horizontal advance values, nor both horizontal and vertical movement in the
-same font, and an OpenType font is now desirable because Type 1 fonts are
-becoming obsolete.
+from strings in this font. Annoyingly, TrueType/OpenType fonts do not permit
+negative horizontal advance values, nor both horizontal and vertical movement
+in the same font, and a Truetype or OpenType font is now desirable because Type
+1 fonts are becoming obsolete.
 
 We get round this restriction by separating out substrings that start with an
 unsupported moving character and consist only of position moving characters.
@@ -754,6 +768,8 @@ ps_string(uint32_t *s, fontinststr *fdata, int32_t *xu, int32_t *yu,
 int32_t x = *xu;
 int32_t y = *yu;
 
+if (ps_changecolour) setcolour();
+
 /* Process the string in substrings in which all the characters have the same
 font, handling the music font specially, as noted above. */
 
@@ -869,6 +885,8 @@ Returns:     nothing
 void
 ps_barline(int32_t x, int32_t ytop, int32_t ybot, int type, int32_t magn)
 {
+if (ps_changecolour) setcolour();
+
 /* Use music font characters if appropriate. */
 
 if (!bar_use_draw &&
@@ -945,6 +963,7 @@ Returns:     nothing
 void
 ps_brace(int32_t x, int32_t ytop, int32_t ybot, int32_t magn)
 {
+if (ps_changecolour) setcolour();
 ps_printf(" %s br%s", SFF("%f %f %f", ((ybot-ytop+16*magn)*23)/12000,
   psxtran(x)+1500, psytran((ytop-16*magn+ybot)/2)),
   (curmovt->bracestyle)? "2":"");
@@ -970,6 +989,7 @@ Returns:     nothing
 void
 ps_bracket(int32_t x, int32_t ytop, int32_t ybot, int32_t magn)
 {
+if (ps_changecolour) setcolour();
 ps_printf("%s", SFF(" %f %f %f k", psxtran(x), psytran(ytop)+16*magn,
   psytran(ybot)));
 }
@@ -1006,6 +1026,9 @@ int ch, i;
 int32_t chwidth = 0;
 int32_t x = leftx;
 
+ps_setgray(0);
+if (ps_changecolour) setcolour();
+
 /* Output the stave using PostScript drawing primitives. */
 
 if (stave_use_draw > 0)
@@ -1107,6 +1130,8 @@ ps_muschar(int32_t x, int32_t y, uint32_t ch, int32_t pointsize)
 {
 int32_t xfudge = 0;
 
+if (ps_changecolour) setcolour();
+
 /* Use a local font data structure which has no rotation. */
 
 ps_mfdata.size = pointsize;
@@ -1159,43 +1184,30 @@ for (mfstr *p = out_mftable[ch]; p != NULL; p = p->next)
 /* There are two versions, one with absolute coordinates, and one with relative
 coordinates, to save having to pass a flag each time (most of the calls are
 with absolute coordinates). The strings are always quite short; we have to
-convert to 32-bits.
+convert to 32-bits by calling string_pmw().
 
 Arguments:
   s          the string
   pointsize  the pointsize for the font
-  x          the absolute x coordinate
-  y          the absolute y coordinate
+  x          the x coordinate
+  y          the y coordinate
 
 Returns:     nothing
 */
 
-static void
-musstring(uschar *s, int32_t pointsize, int32_t x, int32_t y, BOOL absolute)
-{
-ps_mfdata.size = pointsize;
-ps_string(string_pmw(s, font_mf), &ps_mfdata, &x, &y, absolute, FALSE);
-}
-
-
-
 void
 ps_musstring(uschar *s, int32_t pointsize, int32_t x, int32_t y)
 {
-musstring(s, pointsize, x, y, TRUE);
-
-//ps_mfdata.size = pointsize;
-//ps_basic_string(string_pmw(s, 0), font_mf, &ps_mfdata, TRUE, x, y);
+ps_mfdata.size = pointsize;
+ps_string(string_pmw(s, font_mf), &ps_mfdata, &x, &y, TRUE, FALSE);
 }
 
 
 void
 ps_relmusstring(uschar *s, int32_t pointsize, int32_t x, int32_t y)
 {
-musstring(s, pointsize, x, y, FALSE);
-
-//ps_mfdata.size = pointsize;
-//ps_basic_string(string_pmw(s, 0), font_mf, &ps_mfdata, FALSE, x, y);
+ps_mfdata.size = pointsize;
+ps_string(string_pmw(s, font_mf), &ps_mfdata, &x, &y, FALSE, FALSE);
 }
 
 
@@ -1226,6 +1238,8 @@ int32_t depth = -out_stavemagn*((n_fontsize * sign *
     cos(atan((double)beam_slope/1000.0))))/10000)/1000;
 int32_t y0, y1;
 
+if (ps_changecolour) setcolour();
+
 y1 = y0 = out_ystave - beam_firstY +
   mac_muldiv(n_fontsize, (level - 1) * sign * 3 * out_stavemagn, 10000);
 
@@ -1304,6 +1318,8 @@ ps_slur(int32_t x0, int32_t y0, int32_t x1, int32_t y1, uint32_t flags,
 {
 int32_t length = x1 - x0;
 
+if (ps_changecolour) setcolour();
+
 y0 = out_ystave - y0;
 y1 = out_ystave - y1;
 
@@ -1366,6 +1382,8 @@ int32_t dashlength = 0;
 int32_t gaplength = 0;
 int dashcount, spacecount;
 
+if (ps_changecolour) setcolour();
+
 /* Handle "editorial" lines: won't exist if dashed or dotted */
 
 if ((flags & tief_editorial) != 0)
@@ -1442,6 +1460,7 @@ Returns:      nothing
 void
 ps_lines(int32_t *x, int32_t *y, int count, int32_t thickness)
 {
+if (ps_changecolour) setcolour();
 for (int i = count - 1; i > 0; i--)
   ps_printf("%s", SFF(" %f %f", psxtran(x[i]), psytran(out_ystave - y[i])));
 ps_printf(" %d %s", count - 1, SFF("%f %f %f ll", psxtran(x[0]),
@@ -1450,6 +1469,26 @@ ps_printf(" %d %s", count - 1, SFF("%f %f %f ll", psxtran(x[0]),
 
 
 
+/*************************************************
+*         Stroke or fill a path                  *
+*************************************************/
+
+/* Used by ps_path() and ps_abspath() below.
+
+Argument: line thickness for stroke; -1 for fill
+Returns:  nothing
+*/
+
+static void
+strokeorfill(int32_t thickness)
+{
+if (ps_changecolour) setcolour();
+if (thickness >= 0) ps_printf(" %s Slw S", sff(thickness));
+  else ps_printf(" F");
+}
+
+
+
 /*************************************************
 *         Output and stroke or fill a path       *
 *************************************************/
@@ -1489,10 +1528,7 @@ while (*c != path_end) switch(*c++)
   break;
   }
 
-if (ps_gray != 0) ps_printf(" %s Sg", sff(ps_gray));
-if (thickness >= 0) ps_printf(" %s Slw S", sff(thickness));
-  else ps_printf(" F");
-if (ps_gray != 0) ps_printf(" 0 Sg");
+strokeorfill(thickness);
 }
 
 
@@ -1535,34 +1571,63 @@ while (*c != path_end) switch(*c++)
   break;
   }
 
-if (ps_gray != 0) ps_printf(" %s Sg", sff(ps_gray));
-if (thickness >= 0) ps_printf(" %s Slw S", sff(thickness));
-  else ps_printf(" F");
-if (ps_gray != 0) ps_printf(" 0 Sg");
+strokeorfill(thickness);
 }
 
 
 
 /*************************************************
-*            Set gray level                      *
+*            Set gray level or colour            *
 *************************************************/
 
-/* All that happens here is that the gray level is remembered for later use.
+/* All that happens here is that the gray level or colour is remembered for
+later use.
 
-Argument:  the gray level
+Argument:  the colour or the gray level
 Returns:   nothing
 */
 
+void
+ps_setcolour(int32_t *colour)
+{
+memcpy(ps_wantcolour, colour, 3 * sizeof(int32_t));
+ps_changecolour =
+    ps_wantcolour[0] != ps_curcolour[0] ||
+    ps_wantcolour[1] != ps_curcolour[1] ||
+    ps_wantcolour[2] != ps_curcolour[2];
+}
+
+
 void
 ps_setgray(int32_t gray)
 {
-ps_gray = gray;
+int temp[3];
+temp[0] = temp[1] = temp[2] = gray;
+ps_setcolour(temp);
 }
 
 
 
 /*************************************************
-*            Set dash and capandjoin             *
+*            Retrieve current colour             *
+*************************************************/
+
+/* Used to preserve the colour over a drawing.
+
+Argument: pointer to where to put the colour values
+Returns:  nothing
+*/
+
+void
+ps_getcolour(int32_t *colour)
+{
+memcpy(colour, ps_wantcolour, 3 * sizeof(int32_t));
+}
+
+
+
+/*************************************************
+*                   Set dash                     *
 *************************************************/
 
 /* The set values are remembered so that repetition is avoided.
@@ -1570,13 +1635,12 @@ ps_gray = gray;
 Arguments:
   dashlength    the dash length
   gaplength     the gap length
-  caj           the cap-and-join value
 
 Returns:        nothing
 */
 
 void
-ps_setdash(int32_t dashlength, int32_t gaplength, uint32_t caj)
+ps_setdash(int32_t dashlength, int32_t gaplength)
 {
 if (dashlength != out_dashlength || gaplength != out_dashgaplength)
   {
@@ -1585,7 +1649,23 @@ if (dashlength != out_dashlength || gaplength != out_dashgaplength)
   out_dashlength = dashlength;
   out_dashgaplength = gaplength;
   }
+}
 
+
+
+/*************************************************
+*                Set cap and join                *
+*************************************************/
+
+/* The set value is remembered so that repetition is avoided.
+
+Argument: the cap and join flag bits
+Returns:  nothing
+*/
+
+void
+ps_setcapandjoin(uint32_t caj)
+{
 if (caj != ps_caj)
   {
   if ((caj & caj_round) != 0) ps_printf(" 1 Slc");
@@ -2005,7 +2085,7 @@ for (int i = 0; i < font_tablen; i++)
     fontstr *f = font_list + font_table[i];
     if (f->encoding != NULL)
       {
-      for (int k = 0; k < 512; k += 256)
+      for (int k = 0; k < FONTWIDTHS_SIZE; k += 256)
         {
         uschar name[128];
         sprintf(CS name, "%sEnc%c", f->name, (k == 0)? 'L':'U');
