@@ -1,8 +1,10 @@
 /*************************************************
-*     Command Line Decoding Routine (rdargs)     *
+*     Command Line Decoding Function (rdargs)    *
 *************************************************/
 
-/* Header for rdargs routine */
+/* Header for rdargs() function. */
+
+/* This file last edited: January 2021 */
 
 typedef struct {
   int presence;
@@ -12,6 +14,6 @@ typedef struct {
 
 enum { arg_present_not, arg_present_unkeyed, arg_present_keyed };
 
-extern int rdargs(int, char **, const char *, arg_result *);
+extern int rdargs(int, char **, const char *, arg_result *, int size);
 
 /* End of rdargs.h */
