@@ -4,7 +4,7 @@
 
 /* Copyright Philip Hazel 2021 */
 /* This file created: June 2021 */
-/* This file last modified: July 2021 */
+/* This file last modified: October 2023 */
 
 #include "pmw.h"
 
@@ -497,6 +497,22 @@ uint8_t taildirflags[MAX_BEAMNOTES];
 beam_firstX += n_cueadjust;
 xright += n_cueadjust;
 
+/* Adjust for centralized stems */
+
+if ((beam_first->flags & nf_stemcent) != 0)
+  {
+  beam_firstX += (beam_upflag? -1 : +1) *
+    mac_muldiv(STEMCENTADJUST + ((beam_first->noteheadstyle == nh_circular)?
+      -900 : 0), out_stavemagn, 1000);
+  }
+
+if ((beam_last->flags & nf_stemcent) != 0)
+  {
+  xright += (beam_upflag? -1 : +1) *
+    mac_muldiv(STEMCENTADJUST + ((beam_first->noteheadstyle == nh_circular)?
+      -900 : 0), out_stavemagn, 1000);
+  }
+
 /* Adjust parameters for cue or grace notes */
 
 if (n_fontsize != 10000)
@@ -545,7 +561,7 @@ if (beam_accrit != 0)
   return;
   }
 
-/* "Normal" beams - first draw beams which go right through all notes */
+/* "Normal" beams - first draw beams that go right through all notes */
 
 for (int i = 1; i <= throughlevel; i++)
   {
@@ -724,6 +740,16 @@ for (int level = throughlevel + 1; level <= 4; level++)
         if (left_up && left->abspitch != 0) XLadjust = beam_Xcorrection;
         if (right_up || right->spitch == 0) XRadjust = beam_Xcorrection;
 
+        /* Hack for centred stems */
+
+        if ((left->flags & nf_stemcent) != 0)
+          XLadjust += mac_muldiv(left_up? -STEMCENTADJUST : +STEMCENTADJUST,
+            out_stavemagn, 1000);
+
+        if ((right->flags & nf_stemcent) != 0)
+          XRadjust += mac_muldiv(right_up? -STEMCENTADJUST : +STEMCENTADJUST,
+            out_stavemagn, 1000);
+
         /* If the right note is a rest, we must be at the end of a beam,
         beaming over a rest, so make the right hand end the same as the end of
         the main beams. */
@@ -758,6 +784,12 @@ for (int level = throughlevel + 1; level <= 4; level++)
           int32_t x = out_findXoffset(leftmoff) + adjusts[adjustptr] +
             sladjust + (left_up? beam_Xcorrection : 0);
 
+          /* Hack for centred stems */
+
+          if ((left->flags & nf_stemcent) != 0)
+            x += mac_muldiv(left_up? -STEMCENTADJUST : +STEMCENTADJUST,
+              out_stavemagn, 1000);
+
           if (left_up != beam_upflag) Dlevel = throughlevel - level + 1;
           ps_beam(x, x + mac_muldiv(curmovt->beamflaglength, out_stavemagn,
             1000), Dlevel, 0);
@@ -777,6 +809,12 @@ for (int level = throughlevel + 1; level <= 4; level++)
           int32_t x = out_findXoffset(rightmoff) + adjusts[rightadjustptr] +
             sradjust + (right_up? beam_Xcorrection : 0);
 
+          /* Hack for centred stems */
+
+          if ((right->flags & nf_stemcent) != 0)
+            x += mac_muldiv(right_up? -STEMCENTADJUST : +STEMCENTADJUST,
+              out_stavemagn, 1000);
+
           if (right_up != beam_upflag) Dlevel = throughlevel - level + 1;
           ps_beam(x - mac_muldiv(curmovt->beamflaglength, out_stavemagn, 1000),
             x, Dlevel, 0);
