@@ -4,7 +4,7 @@
 
 /* Copyright Philip Hazel 2021 */
 /* This file created: June 2021 */
-/* This file last modified: November 2021 */
+/* This file last modified: October 2023 */
 
 #include "pmw.h"
 
@@ -40,15 +40,16 @@ static uschar *tailstrings[] = {
   US"E", US";v|;xv", US";v|;v|;xvxv", US";v|;v|;v|;xvxvxv" };
 
 /* This table must be kept instep with the definitions of notehead types
-(nh_xx) in pmw.h. */
+(nh_xx) in pmw.h. Each line specifies the noteheads for each type of note,
+starting with breve. */
 
 static uschar headchars[] = {
   '1', '2', 'M', 'L', 'L', 'L', 'L', 'L',    /* normal */
   'n', 'n', 'n', 'n', 'n', 'n', 'n', 'n',    /* cross */
   'm', 'm', 'm', 'l', 'l', 'l', 'l', 'l',    /* harmonic */
    0,   0,   0,   0,   0,   0,   0,   0,     /* none */
-  '1', '2', 'M', 'L', 'L', 'L', 'L', 'L',    /* only = normal */
-  178, 178, 178, 178, 178, 178, 178, 178     /* direct */
+  178, 178, 178, 178, 178, 178, 178, 178,    /* direct */
+  201, 201, 201, 200, 200, 200, 200, 200     /* circular */
 };
 
 /* Table giving musical character identifiers for accidentals, plain,
@@ -308,22 +309,16 @@ uschar *p;
 int32_t fontsize = (n_fontsize*out_stavemagn)/1000;
 int32_t y = out_ystave - (n_pitch - P_1S)*out_pitchmagn - n_pcorrection;
 
-int noteheadstyle;
 int top = P_6L;
 int bot = P_0L;
 
 BOOL ledgered = FALSE;
 BOOL positioned = FALSE;
 BOOL inverted = (n_flags & nf_invert) != 0;
+BOOL stemcent = (n_flags & nf_stemcent) != 0;
 
 TRACE("show_note() start\n");
 
-/* Set the notehead style */
-
-noteheadstyle = ((n_flags & nf_nhharmonic) != 0)? nh_harmonic :
-                ((n_flags & nf_nhcross) != 0)? nh_cross :
-                bar_cont->noteheadstyle;
-
 /* Set up for coupled notes */
 
 if ((n_flags & nf_coupleU) != 0)
@@ -340,7 +335,7 @@ else if ((n_flags & nf_coupleD) != 0)
 /* First deal with ledger lines if required for 5- or 6-line staves. We can
 optimize into a single music-font string if the size is standard. */
 
-if (out_stavelines >= 5 && noteheadstyle != nh_none)
+if (out_stavelines >= 5 && n_noteheadstyle != nh_none)
   {
   int ledgergap;
   int toporbot;
@@ -443,8 +438,9 @@ where there is a complete character available in the music font. */
 
 p = buff;  /* Pointer for generating music font string. */
 
-if (n_notetype < dsquaver && n_stemlength == 0 && noteheadstyle == nh_normal &&
-    (n_flags & (nf_invert|nf_stem|nf_smallhead)) == nf_stem)
+if (n_notetype < dsquaver && n_stemlength == 0 &&
+    n_noteheadstyle == nh_normal && !n_smallhead && !stemcent &&
+    (n_flags & (nf_invert|nf_stem)) == nf_stem)
   {
   if ((n_flags & nf_appogg) != 0) *p++ = n_upflag? 129 : 130;
   *p++ = common_notes[n_notetype + n_upflag*6];
@@ -470,17 +466,120 @@ if ((n_flags & nf_stem) != 0)
   positioned = TRUE;
   if ((n_flags & nf_appogg) != 0) *p++ = n_upflag? 129 : 130;
 
+  /* The variable yy is equal to the note's y coordinate adjusted for any
+  change in stemlength. A string of music characters is constructed to be
+  output at this y coordinate, starting with any tails, followed by stem and
+  up/down movements. Afterwards, there may be a final stem character to be
+  output at the base y coordinate. */
+
   yy = y + ((n_upflag? -1 : +1) * n_stemlength * out_stavemagn)/1000;
   p += sprintf(CS p, "%s",
     tailstrings[n_notetype + (n_upflag? NOTETYPE_COUNT : 0)]);
 
+
+  /* ===================================================================== */
+  /* Handle experimental centred stems separately so as not to pollute the
+  normal stem handling. */
+
+  if (stemcent)
+    {
+    BOOL nhcross = n_noteheadstyle == nh_cross;
+    BOOL nhharm = n_noteheadstyle == nh_harmonic;
+    BOOL nhcirc = n_noteheadstyle == nh_circular;
+    int32_t centx = x + (n_upflag? -1:+1) * mac_muldiv(
+      ((n_noteheadstyle == nh_circular)? -900 : 0) + STEMCENTADJUST,
+      out_stavemagn, 1000);
+
+    if (n_upflag)
+      {
+      if (yy <= y)    /* Stem is standard or lengthened */
+        {
+        int stemch = nhcross? 'J' : 'o';
+        int32_t z = yy;
+        while (z <= y)
+          {
+          p += sprintf(CS p, "%cww|", stemch);
+          z += font10;
+          }
+        p -= 3;
+        *p = 0;
+        ps_musstring(buff, fontsize, centx, yy);
+        p = buff;
+        if (z < y + font10) *p++ = stemch;
+        }
+
+      else            /* Stem is shortened */
+        {
+        int32_t z = yy - font10 - font2;
+        p += sprintf(CS p, "xxx");
+        if (nhharm || (nhcirc && !out_beaming)) z += font2;
+        while (z <= y)
+          {
+          p += sprintf(CS p, "q|");
+          z += font2;
+          }
+        *(--p) = 0;
+        ps_musstring(buff, fontsize, centx, yy);
+        p = buff;
+        }
+      }
+
+    else  /* Stem down */
+      {
+      if (yy >= y)    /* Stem is standard or lengthened */
+        {
+        int stemch = nhcross? 'K' : 'p';
+        int32_t z = yy;
+        while (z >= y)
+          {
+          p += sprintf(CS p, "%cxx~", stemch);
+          z -= font10;
+          }
+        p -= 3;
+        *p = 0;
+        ps_musstring(buff, fontsize, centx, yy);
+        p = buff;
+        if (z > y - font10) *p++ = stemch;
+        }
+
+      else            /* Stem is shortened */
+        {
+        int32_t z = yy + font10 + font2;
+        p += sprintf(CS p, "www");
+        if (nhharm || (nhcirc && !out_beaming)) z -= font2;
+        while (z >= y)
+          {
+          p += sprintf(CS p, "r~v");
+          z -= font1;
+          }
+        *(--p) = 0;
+        ps_musstring(buff, fontsize, centx, yy);
+        p = buff;
+        }
+      }
+
+    /* Output any additional bits at the centralized x coordinate and the base
+    y coordinate. */
+
+    if (p != buff)
+      {
+      *p = 0;
+      ps_musstring(buff, fontsize, centx, y);
+      p = buff;
+      }
+    }
+
+  /* End of handling centred stems -- deal with traditional stems. */
+  /* ===================================================================== */
+
+
   /* Notes with stems up */
 
-  if (n_upflag)
+  else if (n_upflag)
     {
-    if (yy <= y)    /* stem is lengthened */
+    if (yy <= y)    /* Stem is standard or lengthened */
       {
-      int stemch = (noteheadstyle == nh_cross)? 'o' : 'J';
+      int stemch = (n_noteheadstyle == nh_cross)? 'o' : 'J';
       int32_t z = yy;
       while (z <= y)
         {
@@ -492,7 +591,7 @@ if ((n_flags & nf_stem) != 0)
       ps_musstring(buff, fontsize, x, yy);
       p = buff;
       if (z < y + font10) *p++ = stemch;
-      if (noteheadstyle == nh_harmonic) *p++ = 'q';
+      if (n_noteheadstyle == nh_harmonic) *p++ = 'q';
       }
 
     else            /* stem is shortened */
@@ -515,9 +614,9 @@ if ((n_flags & nf_stem) != 0)
 
   else
     {
-    if (yy >= y)    /* stem is lengthened */
+    if (yy >= y)    /* Stem is standard or lengthened */
       {
-      int stemch = (noteheadstyle == nh_cross)? 'p' : 'K';
+      int stemch = (n_noteheadstyle == nh_cross)? 'p' : 'K';
       int32_t z = yy;
       while (z >= y)
         {
@@ -529,7 +628,7 @@ if ((n_flags & nf_stem) != 0)
       ps_musstring(buff, fontsize, x, yy);
       p = buff;
       if (z > y - font10) *p++ = stemch;
-      if (noteheadstyle == nh_harmonic) *p++ = 'r';
+      if (n_noteheadstyle == nh_harmonic) *p++ = 'r';
       }
 
     else            /* stem is shortened */
@@ -551,7 +650,7 @@ if ((n_flags & nf_stem) != 0)
 
 /* Now add the note head */
 
-if (noteheadstyle != nh_none)
+if (n_noteheadstyle != nh_none)
   {
   if (inverted)
     {
@@ -574,12 +673,12 @@ if (noteheadstyle != nh_none)
       }
     }
 
-  /* The special case of a small note head is dealt with below; just omit
-  the note head at this point. */
+  /* The special cases of a small note head or a circular note head are dealt
+  with below; just omit the note head at this point. */
 
-  if ((n_flags & nf_smallhead) == 0)
+  if (!n_smallhead && n_noteheadstyle != nh_circular)
     {
-    *p++ = headchars[n_notetype + NOTETYPE_COUNT*noteheadstyle];
+    *p++ = headchars[n_notetype + NOTETYPE_COUNT*n_noteheadstyle];
 
     /* When printing right-to-left, we put some redundant spacing *after*
     inverted noteheads. This is just a fudge to fool the x-coordinate adjusting
@@ -595,18 +694,19 @@ if (noteheadstyle != nh_none)
 *p = 0;
 ps_musstring(buff, fontsize, x, y);
 
-/* In the special case of a small note head, the note head was skipped above.
-The printing position should be in the correct place for a full size note head
-if a stem or ledger lines were output above. Arrange to output the notehead at
-the cue size, with a relative position adjusted to allow for the head size. */
+/* In the special cases of a small or circular note head, the note head was
+skipped above. The printing position should be in the correct place for a full
+size note head if a stem or ledger lines were output above. Arrange to output
+the notehead at the cue size, with a relative position adjusted to allow for
+the head size. */
 
-if ((n_flags & nf_smallhead) != 0)
+if (n_smallhead)
   {
   int32_t cue_fontsize = (curmovt->fontsizes)->fontsize_cue.size;
   int32_t sm_fontsize = (cue_fontsize * out_stavemagn)/1000;
 
   p = buff;
-  *p++ = headchars[n_notetype + NOTETYPE_COUNT*noteheadstyle];
+  *p++ = headchars[n_notetype + NOTETYPE_COUNT*n_noteheadstyle];
   *p = 0;
 
   if (positioned && (n_flags & nf_stem) != 0)
@@ -627,6 +727,23 @@ if ((n_flags & nf_smallhead) != 0)
     }
   }
 
+else if (n_noteheadstyle == nh_circular)
+  {
+  p = buff;
+  *p++ = headchars[n_notetype + NOTETYPE_COUNT*n_noteheadstyle];
+  *p = 0;
+
+  x += mac_muldiv((n_upflag? 300 : -1200), out_stavemagn, 1000);
+
+  ps_musstring(buff, fontsize, x, y);
+
+
+
+
+
+  }
+
+
 /* Return TRUE if there were ledger lines. */
 
 TRACE("show_note() end\n");
@@ -905,7 +1022,7 @@ if ((bar_cont->flags & cf_notes) != 0 && (n_flags & nf_hidden) == 0)
 
     if (n_notetype == breve) adjustket += 5*out_stavemagn;
 
-    if ((n_flags & nf_smallhead) != 0)
+    if (n_smallhead)
       {
       if ((n_flags & nf_stem) != 0)
         {
@@ -1480,6 +1597,8 @@ n_flags = n_chordflags = p->flags;
 n_lastnote = p;
 n_length = p->length;
 n_masq = p->masq;
+n_noteheadstyle = p->noteheadstyle & nh_mask;
+n_smallhead = (p->noteheadstyle & nhf_smallhead) != 0;
 n_notetype = p->notetype;
 n_pitch = p->spitch;
 
@@ -1840,7 +1959,7 @@ if (n_notetype >= quaver)
     /* If not beaming, see if this is the start of one, and cause the beam to
     be output if it is. */
 
-    if (!out_beaming && (bar_cont->flags & cf_noteheads) == 0)
+    if (!out_beaming && (n_flags & nf_stem) != 0)
       out_beaming = out_setupbeam(p, out_moff + n_gracemoff, FALSE, FALSE);
 
     /* If beaming, compute the correct stem length and turn the note type into
@@ -1849,8 +1968,10 @@ if (n_notetype >= quaver)
 
     if (out_beaming)
       {
+      int scadjust = ((n_flags & nf_stemcent) == 0)? 0 :
+        n_upflag? -STEMCENTADJUST : +STEMCENTADJUST;
       n_stemlength = (n_upfactor*(beam_firstY + mac_muldiv(beam_slope,
-        out_findXoffset(out_moff+n_gracemoff) + out_Xadjustment +
+        out_findXoffset(out_moff+n_gracemoff) + out_Xadjustment + scadjust +
           (75*out_stavemagn)/100 +
             (n_upflag? beam_Xcorrection : 0) -
               beam_firstX, 1000) -
@@ -2332,6 +2453,8 @@ if (n_chordcount > 1)
     n_acflags = tp->acflags;
     n_acc = tp->acc;
     n_accleft = (tp->accleft*out_stavemagn)/1000;
+    n_noteheadstyle = tp->noteheadstyle & nh_mask;
+    n_smallhead = (tp->noteheadstyle & nhf_smallhead) != 0;
 
     mac_couplepitch(n_pitch, n_flags);
     n_stemlength = (abs(n_pitch - lastpitch)/2 - (14*n_fontsize)/10000) * 1000;
