@@ -4,7 +4,7 @@
 
 /* Copyright Philip Hazel 2022 */
 /* This file created: January 2021 */
-/* This file last modified: July 2022 */
+/* This file last modified: December 2023 */
 
 #include "pmw.h"
 
@@ -1077,26 +1077,26 @@ for (uint32_t *s = str; *s != 0; s++)
   f = PFONT(*s) & ~font_small;
   fs = &(font_list[font_table[f]]);
 
-  /* For a non-standardly encoded font, see if there's a Unicode translation.
-  If not, a value greater than 255 is erroneous, unless there is a custom
-  encoding, in which case values can go up to 511. Remember invalid code
-  points, to be listed at the end of reading the input, and replace with
-  whatever is set for the font, retaining the font number if the replacement's
-  font is unset. */
+  /* For non-standardly encoded fonts, either the character or its translation
+  must be valid. */
 
   if ((fs->flags & ff_stdencoding) == 0)
     {
-    uint32_t climit = (fs->encoding == NULL)? 256 : 512;
-    uint32_t pc = font_utranslate(c, fs);
-    *s &= 0xff000000u;   /* Keep the current font */
+    uint32_t tc = font_utranslate(c, fs);
 
-    /* A valid translated codepoint is either < climit or an escaped sharp,
-    equals, or hyphen, whose code points are outside the Unicode range. */
+    if (tc == 0xffffffffu) tc = c;   /* No translation found */
+    *s &= 0xff000000u;               /* Keep the current font */
+
+    /* A valid, possibly translated, codepoint is either less than 512 or an
+    escaped sharp, equals, or hyphen, whose code points are outside the Unicode
+    range. */
+
+    if (tc < 512 || tc > MAX_UNICODE) *s |= tc;
+
+    /* Remember invalid code points, to be listed at the end of reading the
+    input, and replace with whatever is set for the font, retaining the font
+    number if the replacement's font is unset. */
 
-    if (pc < climit || pc > MAX_UNICODE)
-      {
-      *s |= pc;
-      }
     else
       {
       *s = (PFTOP(fs->invalid) == (font_unknown << 24))?
@@ -1120,6 +1120,8 @@ for (uint32_t *s = str; *s != 0; s++)
 
   else
     {
+    uint32_t tc;
+
     if (c == '`' || c == '\'' || (c == '\n' && !keepnl) ||
          (c == 'f' && PCHAR(s[1]) == 'i'))
       {
@@ -1136,15 +1138,26 @@ for (uint32_t *s = str; *s != 0; s++)
       *s = PFTOP(*s) | c;
       }
 
-    /* Code points >= 256 and < LOWCHARLIMIT are encoded in the second of the
-    two PostScript fonts, using the Unicode encoding less 256, so no more needs
-    to be done. The remaining code points have to be converted to some of the
+    /* Check for a Unicode translation. If one is found, it will either be less
+    than 512 or an escaped sharp, equals, or hyphen with a high code point.
+    This is checked when the .utr file is read so no need to check here. */
+
+    tc = font_utranslate(c, fs);
+    if (tc != 0xffffffffu)   /* A translation was found */
+      {
+      *s = PFTOP(*s) | tc;
+      }
+
+    /* Handle an untranslated character in a standardly encoded font. Code
+    points >= 256 and < LOWCHARLIMIT are encoded in the second of the two
+    PostScript fonts, using the Unicode encoding less 256, so no more needs to
+    be done. The remaining code points have to be converted to some of the
     remaining characters in the second PostScript font, which are encoded
     arbitrarily, i.e. not using the Unicode encoding (some of their Unicode
     values are quite large). To find this encoding, we search for the character
     in the tree for the font . */
 
-    if (c >= LOWCHARLIMIT)
+    else if (c >= LOWCHARLIMIT)
       {
       tree_node *t;
       uschar utf[8];
@@ -1153,10 +1166,13 @@ for (uint32_t *s = str; *s != 0; s++)
       t = tree_search(fs->high_tree, utf);
       *s &= 0xff000000u;   /* Keep the current font */
 
-      /* If the character is unknown, replace it with whatever is set for the
-      font, retaining the font number if the replacement's font is unset. */
+      /* If the character is unknown, leave it alone if less than 512.
+      Otherwise replace it with whatever is set for the font, retaining the
+      font number if the replacement's font is unset. */
 
-      if (t != NULL) *s |= LOWCHARLIMIT + t->value; else
+      if (t != NULL) *s |= LOWCHARLIMIT + t->value;
+      else if (c < 512) *s |= c;
+      else
         {
         *s = (PFTOP(fs->invalid) == (font_unknown << 24))?
           (*s | PCHAR(fs->invalid)) : fs->invalid;
@@ -1814,7 +1830,7 @@ Arguments:
   b         points to a structure to return data
   rehearse  TRUE for a rehearsal string
   opts      if not NULL, restrict to these options
-  opterror  string for bad option error
+  opterror  string for bad option error when opts not NULL
 
 Returns:   TRUE if the options end with /" (current character is then ")
 */
@@ -1837,7 +1853,7 @@ read_basestring(basestavestring *b, BOOL rehearse, const char *opts,
   const char *opterror)
 {
 b->hadab = FALSE;
-b->flags = srs.textflags;
+b->flags = rehearse? 0 : srs.textflags;
 b->offset = b->adjustx = b->adjusty = b->halfway = b->rotate = 0;
 b->size = -1;
 
@@ -2001,7 +2017,13 @@ while (read_c == '/' && main_readbuffer[read_i] != '/')
     break;
 
     case 'r':
-    if (Ustrncmp(main_readbuffer + read_i, "ing", 3) == 0)
+    if (Ustrncmp(main_readbuffer + read_i, "box", 3) == 0)
+      {
+      b->flags |= text_boxed | text_boxrounded;
+      read_i += 3;
+      read_nextc();
+      }
+    else if (Ustrncmp(main_readbuffer + read_i, "ing", 3) == 0)
       {
       b->flags |= text_ringed;
       read_i += 3;
@@ -2074,7 +2096,7 @@ while (read_c == '/' && main_readbuffer[read_i] != '/')
 
     default:
     error_skip(ERR8, ('/'<<8) | ' ', "/a, /ao, /b, /bar, /box, /bu, /d, /e, "
-      "/F, /fb, /h, /l, /m, /ol, /r, /ring, /S, /s, /u or /ul");
+      "/F, /fb, /h, /l, /m, /ol, /r, /rbox, /ring, /S, /s, /u or /ul");
     break;
     }
   }
@@ -2151,18 +2173,25 @@ basestavestring *s3 = NULL;
 /* Read the basic string. If it ends with /" there is special handling for
 underlay/overlay. */
 
-more = read_basestring(s1, rehearse, NULL,
-  "/a, /ao, /b, /bar, /box, /bu, /d, /e, /F, /fb, "
-  "/h, /l, /m, /ol, /r, /ring, /S, /s, /u or /ul");
-
+more = read_basestring(s1, rehearse, NULL, NULL);
 if (s1->string == NULL) return;    /* There's been an error */
 
-undoverlay = (s1->flags & text_ul) != 0;
+/* Non-movement options and size settings are ignored on rehearsal marks; they
+always use the rehearsal marks style and size. Warn if any are present. */
+
+if (rehearse && (s1->flags != 0 || s1->size >= 0))
+  {
+  error(ERR178);
+  s1->flags = 0;  /* No flags */
+  s1->size = -1;  /* Unset */
+  }
 
 /* Now that we know what type of text this is, we can set up the default font
 for underlay, overlay, figured bass, or general text, and also set the size if
 it is not set. */
 
+undoverlay = (s1->flags & text_ul) != 0;  /* Save repeated testing */
+
 if (undoverlay)
   {
   if ((s1->flags & text_above) == 0)
@@ -2188,9 +2217,7 @@ else if ((s1->flags & text_fb) != 0)
 else if (rehearse)
   {
   default_font = curmovt->fonttype_rehearse;
-  if (s1->size >= 0) error(ERR11, "/s or /S on a rehearsal mark");
   s1->size = 0;
-  s1->flags |= curmovt->rehearsalstyle;
   }
 
 else
@@ -2325,7 +2352,7 @@ if ((s1->flags & text_absolute) != 0 && !s1->hadab)
 
 /* Override flags for rehearsal strings. */
 
-if (rehearse) s1->flags = text_rehearse | text_above;
+if (rehearse) s1->flags = text_rehearse | text_above | curmovt->rehearsalstyle;
 
 /* Set up a stave text block that is not (yet) connected to the current bar's
 chain of items. */
