@@ -4,7 +4,7 @@
 
 /* Copyright Philip Hazel 2022 */
 /* This file created: December 2020 */
-/* This file last modified: July 2022 */
+/* This file last modified: November 2023 */
 
 /* These structures must be defined before the stave data items. */
 
@@ -219,6 +219,7 @@ typedef struct {
   uint16_t spitch;
   uint8_t  acc_orig;
   uint8_t  char_orig;
+  uint8_t  noteheadstyle;
 } b_notestr;
 
 typedef b_bytevaluestr b_noteheadsstr;
@@ -269,6 +270,11 @@ typedef struct {
   int32_t yright;
 } b_pletstr;
 
+typedef struct {
+  BSTRHEAD;
+  int32_t moff;
+} b_resetstr;
+
 typedef struct {
   BSTRHEAD;
   CBOOL relative;
@@ -468,14 +474,14 @@ typedef struct {
   uint16_t  size;
 } drawtextstr;
 
-
 /* Layout of data structure used for saving drawing operations until the
 end of a stave - to ensure they are over everything else. Use a union
 to handle the two different types. */
 
 struct overdrawstr_graphic {
   int32_t linewidth;
-  int32_t gray;
+  int32_t dash[2];
+  int32_t colour[3];
   int32_t ystave;
   int32_t *x;
   int32_t *y;
@@ -485,7 +491,8 @@ struct overdrawstr_graphic {
 struct overdrawstr_text {
   uint32_t *text;
   fontinststr fdata;
-  uint32_t boxring;
+  uint32_t flags;
+  int32_t colour[3];
   int32_t xx;
   int32_t yy;
   int32_t matrix[6];
@@ -621,6 +628,7 @@ typedef struct sreadstr {
   uint16_t   fbsize;
    int16_t   hairpinbegun;
    int16_t   hairpinflags;
+  uint16_t   lasttiepitch;
    int16_t   maxaway;
   uint16_t   maxpitch;
   uint16_t   minpitch;
@@ -637,11 +645,11 @@ typedef struct sreadstr {
   uint8_t    accrit;
    int8_t    beamstemforce;
   uint8_t    chordcount;    /* Number of notes in last note/chord */
+  uint8_t    noteheadstyle;
   uint8_t    ornament;
   uint8_t    pletflags;
    int8_t    stemsdirection;
    int8_t    tiesplacement;
-  uint16_t   lasttiepitch;
   CBOOL      beaming;
   CBOOL      noteson;
   CBOOL      laststemup;
@@ -992,7 +1000,6 @@ typedef struct constr {
   uolaystr   *uolay;           /* pending {und,ov}erlay hyphens or extension */
   int32_t     tiex;            /* start position for tie */
   uint32_t    time;            /* time signature */
-  uint8_t     noteheadstyle;   /* sic */
   uint8_t     flags;           /* see below */
   uint8_t     clef;            /* current clef */
   uint8_t     key;             /* current key signature */
@@ -1001,8 +1008,7 @@ typedef struct constr {
 #define cf_bowingabove   1
 #define cf_notes         2     /* on/off switch */
 #define cf_triplets      4     /* on/off switch */
-#define cf_noteheads     8     /* on/off switch for noteheads only */
-#define cf_rdrepeat     16     /* last bar ended with double repeat */
+#define cf_rdrepeat      8     /* last bar ended with double repeat */
 
 #define cf_default (cf_bowingabove|cf_notes|cf_triplets)
 
